//---------------------------------------------------------------------------
#include <stdio.h>
#include "Allometry.h"
#include "ParsingFunctions.h"

////////////////////////////////////////////////////////////////////////////
// Constructor
// Edit history:
// -------------
// April 25, 2005 - Created (LEM)
////////////////////////////////////////////////////////////////////////////
clAllometry::clAllometry()
{

  m_fMaxStdCrownRad = 10;

  mp_AdultHeight = NULL;
  mp_AdultDiam = NULL;
  mp_AdultCrownRad = NULL;
  mp_AdultCrownHeight = NULL;
  mp_SaplingHeight = NULL;
  mp_SaplingDiam = NULL;
  mp_SaplingCrownRad = NULL;
  mp_SaplingCrownHeight = NULL;
  mp_SeedlingHeight = NULL;
  mp_SeedlingDiam = NULL;
  mp_fMaxTreeHeight = NULL;
  mp_fAsympCrownRad = NULL;
  mp_fCrownRadExp = NULL;
  mp_fDbhToDiam10Slope = NULL;
  mp_fDbhToDiam10Intercept = NULL;
  mp_fAsympCrownHeight = NULL;
  mp_fCrownHeightExp = NULL;
  mp_fSlopeHeightDiam10 = NULL;
  mp_fSlopeAsympHeight = NULL;
  mp_fAdultLinearSlope = NULL;
  mp_fAdultLinearIntercept = NULL;
  mp_fAdultReverseLinearSlope = NULL;
  mp_fAdultReverseLinearIntercept = NULL;
  mp_fSaplingLinearSlope = NULL;
  mp_fSaplingLinearIntercept = NULL;
  mp_fSaplingReverseLinearSlope = NULL;
  mp_fSaplingReverseLinearIntercept = NULL;
  mp_fSeedlingLinearSlope = NULL;
  mp_fSeedlingLinearIntercept = NULL;
  mp_fSeedlingReverseLinearSlope = NULL;
  mp_fSeedlingReverseLinearIntercept = NULL;
  mp_fCRCrownRadIntercept = NULL;
  mp_fCRAsympCrownRad = NULL;
  mp_fCRCrownRadShape1 = NULL;
  mp_fCRCrownRadShape2 = NULL;
  mp_fCRCrownHtIntercept = NULL;
  mp_fCRAsympCrownHt = NULL;
  mp_fCRCrownHtShape1 = NULL;
  mp_fCRCrownHtShape2 = NULL;
  mp_fPowerA = NULL;
  mp_fPowerExpB = NULL;
}

////////////////////////////////////////////////////////////////////////////
// Destructor
// Edit history:
// -------------
// April 25, 2005 - Created (LEM)
////////////////////////////////////////////////////////////////////////////
clAllometry::~clAllometry()
{
  delete[] mp_AdultHeight;
  delete[] mp_AdultDiam;
  delete[] mp_AdultCrownRad;
  delete[] mp_AdultCrownHeight;
  delete[] mp_SaplingHeight;
  delete[] mp_SaplingDiam;
  delete[] mp_SaplingCrownRad;
  delete[] mp_SaplingCrownHeight;
  delete[] mp_SeedlingHeight;
  delete[] mp_SeedlingDiam;
  delete[] mp_fMaxTreeHeight;
  delete[] mp_fAsympCrownRad;
  delete[] mp_fCrownRadExp;
  delete[] mp_fDbhToDiam10Slope;
  delete[] mp_fDbhToDiam10Intercept;
  delete[] mp_fAsympCrownHeight;
  delete[] mp_fCrownHeightExp;
  delete[] mp_fSlopeHeightDiam10;
  delete[] mp_fSlopeAsympHeight;
  delete[] mp_fAdultLinearSlope;
  delete[] mp_fAdultLinearIntercept;
  delete[] mp_fAdultReverseLinearSlope;
  delete[] mp_fAdultReverseLinearIntercept;
  delete[] mp_fSaplingLinearSlope;
  delete[] mp_fSaplingLinearIntercept;
  delete[] mp_fSaplingReverseLinearSlope;
  delete[] mp_fSaplingReverseLinearIntercept;
  delete[] mp_fSeedlingLinearSlope;
  delete[] mp_fSeedlingLinearIntercept;
  delete[] mp_fSeedlingReverseLinearSlope;
  delete[] mp_fSeedlingReverseLinearIntercept;
  delete[] mp_fCRCrownRadIntercept;
  delete[] mp_fCRAsympCrownRad;
  delete[] mp_fCRCrownRadShape1;
  delete[] mp_fCRCrownRadShape2;
  delete[] mp_fCRCrownHtIntercept;
  delete[] mp_fCRAsympCrownHt;
  delete[] mp_fCRCrownHtShape1;
  delete[] mp_fCRCrownHtShape2;
  delete[] mp_fPowerA;
  delete[] mp_fPowerExpB;
}

////////////////////////////////////////////////////////////////////////////
// GetData()
// Edit history:
// -------------
// April 25, 2005 - Created (LEM)
// July 8, 2005 - Added crown shape allometry params for multiple possible
// functions (LEM)
// January 25, 2005 - Added power function parameters (LEM)
////////////////////////////////////////////////////////////////////////////
void clAllometry::GetData( DOMDocument * p_oDoc, clTreePopulation * p_oPop )
{
  floatVal * p_fTemp = NULL; //for extracting values for a subset of all species
  int * p_iWhatFunction = NULL; //for getting which function to use for each
    //species
  try
  {
    DOMElement * p_oElement = p_oDoc->getDocumentElement();
    int i, iHowMany;

    m_iNumSpecies = p_oPop->GetNumberOfSpecies();
    p_iWhatFunction = new int[m_iNumSpecies];

    //Declare our arrays that we'll always need
    mp_AdultHeight = new Ptr2Allometry[m_iNumSpecies];
    mp_AdultDiam = new Ptr2Allometry[m_iNumSpecies];
    mp_AdultCrownRad = new Ptr2Allometry[m_iNumSpecies];
    mp_AdultCrownHeight = new Ptr2Allometry[m_iNumSpecies];
    mp_SaplingHeight = new Ptr2Allometry[m_iNumSpecies];
    mp_SaplingDiam = new Ptr2Allometry[m_iNumSpecies];
    mp_SaplingCrownRad = new Ptr2Allometry[m_iNumSpecies];
    mp_SaplingCrownHeight = new Ptr2Allometry[m_iNumSpecies];
    mp_SeedlingHeight = new Ptr2Allometry[m_iNumSpecies];
    mp_SeedlingDiam = new Ptr2Allometry[m_iNumSpecies];
    mp_fMaxTreeHeight = new float[m_iNumSpecies];
    mp_fDbhToDiam10Slope = new float[m_iNumSpecies];
    mp_fDbhToDiam10Intercept = new float[m_iNumSpecies];

    //Get all-species parameters
    FillSpeciesSpecificValue( p_oElement, "tr_canopyHeight", "tr_chVal", mp_fMaxTreeHeight, p_oPop, true );
    FillSpeciesSpecificValue( p_oElement, "tr_conversionDiam10ToDBH", "tr_cdtdVal", mp_fDbhToDiam10Slope, p_oPop, true );
    FillSpeciesSpecificValue( p_oElement, "tr_interceptDiam10ToDBH", "tr_idtdVal", mp_fDbhToDiam10Intercept, p_oPop, true );

    //If the value of the dbh to diam10 conversion factor is zero, throw error
    for ( int i = 0; i < m_iNumSpecies; i++ )
    {
      if ( mp_fDbhToDiam10Slope[i] == 0 )
      {
        modelErr stcErr;
        strcpy( stcErr.cFunction, "clAllometry::GetData" );
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cMoreInfo, "Diameter at 10 cm to DBH conversion may not equal 0." );
        throw( stcErr );
      }
    }

    //Start by getting the allometry function for each species for each life
    //history stage
    //***********************
    // Adults - height-diameter relationship
    //***********************
    FillSpeciesSpecificValue( p_oElement, "tr_whatAdultHeightDiam", "tr_wahdVal", p_iWhatFunction, p_oPop, true );

    //Use the values to set function pointers - if any are not a valid function
    //enum number, throw an error
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard == p_iWhatFunction[i] )
      {
        mp_AdultHeight[i] = & clAllometry::CalcStandardSapAdultHeight;
        mp_AdultDiam[i] = & clAllometry::CalcStandardSapAdultDbh;
      }
      else if ( linear == p_iWhatFunction[i] )
      {
        mp_AdultHeight[i] = & clAllometry::CalcLinearAdultHeight;
        mp_AdultDiam[i] = & clAllometry::CalcLinearAdultDbh;
      }
      else if ( reverse_linear == p_iWhatFunction[i] )
      {
        mp_AdultHeight[i] = & clAllometry::CalcReverseLinearAdultHeight;
        mp_AdultDiam[i] = & clAllometry::CalcReverseLinearAdultDbh;
      }
      else
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clAllometry::GetData" );
        sprintf( stcErr.cMoreInfo, "%s%d", "Invalid adult height-diam function code: ", p_iWhatFunction[i] );
        throw( stcErr );
      }
    }

    //If any of the functions are standard, read standard parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( standard == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare the arrays we'll need
      if (!mp_fSlopeAsympHeight) {
        mp_fSlopeAsympHeight = new float[m_iNumSpecies];
      }

      FillSpeciesSpecificValue( p_oElement, "tr_slopeOfAsymHeight", "tr_soahVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fSlopeAsympHeight[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //If any of the functions are linear, read linear parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( linear == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( linear == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare the arrays we'll need
      mp_fAdultLinearSlope = new float[m_iNumSpecies];
      mp_fAdultLinearIntercept = new float[m_iNumSpecies];

      //Linear adult slope - throw an error if any values are 0
      FillSpeciesSpecificValue( p_oElement, "tr_adultLinearSlope", "tr_alsVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        if ( 0 == p_fTemp[i].val )
        {
          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy( stcErr.cFunction, "clAllometry::GetData" );
          strcpy( stcErr.cMoreInfo, "No values in adult linear slope may be 0." );
          throw( stcErr );

        }
        mp_fAdultLinearSlope[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Linear adult intercept
      FillSpeciesSpecificValue( p_oElement, "tr_adultLinearIntercept", "tr_aliVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fAdultLinearIntercept[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //If any of the functions are reverse linear, read linear parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( reverse_linear == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( reverse_linear == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      mp_fAdultReverseLinearSlope = new float[m_iNumSpecies];
      mp_fAdultReverseLinearIntercept = new float[m_iNumSpecies];

      //Reverse linear adult slope - throw an error if any are 0
      FillSpeciesSpecificValue( p_oElement, "tr_adultReverseLinearSlope", "tr_arlsVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        if ( 0 == p_fTemp[i].val )
        {
          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy( stcErr.cFunction, "clAllometry::GetData" );
          strcpy( stcErr.cMoreInfo, "No values in adult reverse linear slope may be 0." );
          throw( stcErr );

        }
        mp_fAdultReverseLinearSlope[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Reverse linear adult intercept
      FillSpeciesSpecificValue( p_oElement, "tr_adultReverseLinearIntercept", "tr_arliVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fAdultReverseLinearIntercept[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //***********************
    // Adults - crown radius-diameter relationship
    //***********************
    FillSpeciesSpecificValue( p_oElement, "tr_whatAdultCrownRadDiam", "tr_wacrdVal", p_iWhatFunction, p_oPop, true );

    //Use the values to set function pointers - if any are not a valid function
    //enum number, throw an error
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard_cr == p_iWhatFunction[i] )
      {
        mp_AdultCrownRad[i] = & clAllometry::CalcStandardSapAdultCrownRad;
      }
      else if ( chapman_richards_cr == p_iWhatFunction[i] )
      {
        mp_AdultCrownRad[i] = & clAllometry::CalcChapRichSapAdultCrownRad;
      }
      else
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clAllometry::GetData" );
        sprintf( stcErr.cMoreInfo, "%s%d", "Invalid adult crown radius-diam function code: ", p_iWhatFunction[i] );
        throw( stcErr );
      }
    }

    //If any of the functions are standard, read standard parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard_cr == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( standard == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare arrays we'll need
      if (!mp_fAsympCrownRad)
        mp_fAsympCrownRad = new float[m_iNumSpecies];
      if (!mp_fCrownRadExp)
        mp_fCrownRadExp = new float[m_iNumSpecies];

      FillSpeciesSpecificValue( p_oElement, "tr_stdAsympCrownRad", "tr_sacrVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fAsympCrownRad[p_fTemp[i].code] = p_fTemp[i].val;
      }

      FillSpeciesSpecificValue( p_oElement, "tr_stdCrownRadExp", "tr_screVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCrownRadExp[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //If any of the functions are Chapman-Richards, read parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( chapman_richards_cr == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( linear == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare the arrays we'll need
      if (!mp_fCRCrownRadIntercept)
        mp_fCRCrownRadIntercept = new float[m_iNumSpecies];
      if (!mp_fCRAsympCrownRad)
        mp_fCRAsympCrownRad = new float[m_iNumSpecies];
      if (!mp_fCRCrownRadShape1)
        mp_fCRCrownRadShape1 = new float[m_iNumSpecies];
      if (!mp_fCRCrownRadShape2)
        mp_fCRCrownRadShape2 = new float[m_iNumSpecies];

      //Crown radius intercept
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownRadIntercept", "tr_crcriVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRCrownRadIntercept[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Asymptotic crown radius
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownRadAsymp", "tr_crcraVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRAsympCrownRad[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Shape parameter 1
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownRadShape1b", "tr_crcrs1bVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRCrownRadShape1[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Shape parameter 2
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownRadShape2c", "tr_crcrs2cVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRCrownRadShape2[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //***********************
    // Adults - crown height-height relationship
    //***********************
    FillSpeciesSpecificValue( p_oElement, "tr_whatAdultCrownHeightHeight", "tr_wachhVal", p_iWhatFunction, p_oPop, true );

    //Use the values to set function pointers - if any are not a valid function
    //enum number, throw an error
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard_cr == p_iWhatFunction[i] )
      {
        mp_AdultCrownHeight[i] = & clAllometry::CalcStandardSapAdultCrownHeight;
      }
      else if ( chapman_richards_cr == p_iWhatFunction[i] )
      {
        mp_AdultCrownHeight[i] = & clAllometry::CalcChapRichSapAdultCrownHeight;
      }
      else
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clAllometry::GetData" );
        sprintf( stcErr.cMoreInfo, "%s%d", "Invalid adult crown height-diam function code: ", p_iWhatFunction[i] );
        throw( stcErr );
      }
    }

    //If any of the functions are standard, read standard parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard_cr == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( standard == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare arrays we'll need
      if (!mp_fAsympCrownHeight)
        mp_fAsympCrownHeight = new float[m_iNumSpecies];
      if (!mp_fCrownHeightExp)
        mp_fCrownHeightExp = new float[m_iNumSpecies];

      FillSpeciesSpecificValue( p_oElement, "tr_stdAsympCrownHt", "tr_sachVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fAsympCrownHeight[p_fTemp[i].code] = p_fTemp[i].val;
      }

      FillSpeciesSpecificValue( p_oElement, "tr_stdCrownHtExp", "tr_scheVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCrownHeightExp[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //If any of the functions are Chapman-Richards, read parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( chapman_richards_cr == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( linear == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare any arrays we'll need
      if (!mp_fCRCrownHtIntercept)
        mp_fCRCrownHtIntercept = new float[m_iNumSpecies];
      if (!mp_fCRAsympCrownHt)
        mp_fCRAsympCrownHt = new float[m_iNumSpecies];
      if (!mp_fCRCrownHtShape1)
        mp_fCRCrownHtShape1 = new float[m_iNumSpecies];
      if (!mp_fCRCrownHtShape2)
        mp_fCRCrownHtShape2 = new float[m_iNumSpecies];

      //Crown height intercept
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownHtIntercept", "tr_crchiVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRCrownHtIntercept[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Asymptotic crown height
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownHtAsymp", "tr_crchaVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRAsympCrownHt[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Shape parameter 1
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownHtShape1b", "tr_crchs1bVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRCrownHtShape1[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Shape parameter 2
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownHtShape2c", "tr_crchs2cVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRCrownHtShape2[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }


    //***********************
    // Saplings - height diameter
    //***********************
    FillSpeciesSpecificValue( p_oElement, "tr_whatSaplingHeightDiam", "tr_wsahdVal", p_iWhatFunction, p_oPop, true );

    //Use the values to set function pointers - if any are not a valid function
    //enum number, throw an error
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard == p_iWhatFunction[i] )
      {
        mp_SaplingHeight[i] = & clAllometry::CalcStandardSapAdultHeight;
        mp_SaplingDiam[i] = & clAllometry::CalcStandardSapAdultDbh;
      }
      else if ( linear == p_iWhatFunction[i] )
      {
        mp_SaplingHeight[i] = & clAllometry::CalcLinearSaplingHeight;
        mp_SaplingDiam[i] = & clAllometry::CalcLinearSaplingDbh;
      }
      else if ( reverse_linear == p_iWhatFunction[i] )
      {
        mp_SaplingHeight[i] = & clAllometry::CalcReverseLinearSaplingHeight;
        mp_SaplingDiam[i] = & clAllometry::CalcReverseLinearSaplingDbh;
      }
      else if ( power == p_iWhatFunction[i] )
      {
        mp_SaplingHeight[i] = & clAllometry::CalcPowerSaplingHeight;
        mp_SaplingDiam[i] = & clAllometry::CalcPowerSaplingDbh;
      }
      else
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clAllometry::GetData" );
        sprintf( stcErr.cMoreInfo, "%s%d", "Invalid sapling height-diam function code: ", p_iWhatFunction[i] );
        throw( stcErr );
      }
    }

    //If any of the functions are standard, read standard parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( standard == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      if (!mp_fSlopeAsympHeight) {
        mp_fSlopeAsympHeight = new float[m_iNumSpecies];
      }

      FillSpeciesSpecificValue( p_oElement, "tr_slopeOfAsymHeight", "tr_soahVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fSlopeAsympHeight[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //If any of the functions are linear, read linear parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( linear == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( linear == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare arrays we'll need
      mp_fSaplingLinearSlope = new float[m_iNumSpecies];
      mp_fSaplingLinearIntercept = new float[m_iNumSpecies];

      //Linear sapling slope - throw an error if any values are 0
      FillSpeciesSpecificValue( p_oElement, "tr_saplingLinearSlope", "tr_salsVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        if ( 0 == p_fTemp[i].val )
        {
          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy( stcErr.cFunction, "clAllometry::GetData" );
          strcpy( stcErr.cMoreInfo, "No values in sapling linear slope may be 0." );
          throw( stcErr );

        }
        mp_fSaplingLinearSlope[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Linear sapling intercept
      FillSpeciesSpecificValue( p_oElement, "tr_saplingLinearIntercept", "tr_saliVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fSaplingLinearIntercept[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //If any of the functions are reverse linear, read reverse linear
    //parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( reverse_linear == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( reverse_linear == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare arrays we'll need
      mp_fSaplingReverseLinearSlope = new float[m_iNumSpecies];
      mp_fSaplingReverseLinearIntercept = new float[m_iNumSpecies];

      //Reverse linear sapling slope - throw an error if any values are 0
      FillSpeciesSpecificValue( p_oElement, "tr_saplingReverseLinearSlope", "tr_sarlsVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        if ( 0 == p_fTemp[i].val )
        {
          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy( stcErr.cFunction, "clAllometry::GetData" );
          strcpy( stcErr.cMoreInfo, "No values in sapling reverse linear slope may be 0." );
          throw( stcErr );

        }
        mp_fSaplingReverseLinearSlope[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Reverse linear sapling intercept
      FillSpeciesSpecificValue( p_oElement, "tr_saplingReverseLinearIntercept", "tr_sarliVal", p_fTemp,
           iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fSaplingReverseLinearIntercept[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //If any of the functions are power, read power function parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( power == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( power == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare arrays we'll need
      mp_fPowerA = new float[m_iNumSpecies];
      mp_fPowerExpB = new float[m_iNumSpecies];

      //Power function "a"
      FillSpeciesSpecificValue( p_oElement, "tr_saplingPowerA", "tr_sapaVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fPowerA[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Power function "b"
      FillSpeciesSpecificValue( p_oElement, "tr_saplingPowerB", "tr_sapbVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
         mp_fPowerExpB[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //***********************
    // Saplings - crown radius-diameter relationship
    //***********************
    FillSpeciesSpecificValue( p_oElement, "tr_whatSaplingCrownRadDiam", "tr_wscrdVal", p_iWhatFunction, p_oPop, true );

    //Use the values to set function pointers - if any are not a valid function
    //enum number, throw an error
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard_cr == p_iWhatFunction[i] )
      {
        mp_SaplingCrownRad[i] = & clAllometry::CalcStandardSapAdultCrownRad;
      }
      else if ( chapman_richards_cr == p_iWhatFunction[i] )
      {
        mp_SaplingCrownRad[i] = & clAllometry::CalcChapRichSapAdultCrownRad;
      }
      else
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clAllometry::GetData" );
        sprintf( stcErr.cMoreInfo, "%s%d", "Invalid sapling crown radius-diam function code: ", p_iWhatFunction[i] );
        throw( stcErr );
      }
    }

    //If any of the functions are standard, read standard parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard_cr == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( standard == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare arrays we'll need
      if (!mp_fAsympCrownRad)
        mp_fAsympCrownRad = new float[m_iNumSpecies];
      if (!mp_fCrownRadExp)
        mp_fCrownRadExp = new float[m_iNumSpecies];

      FillSpeciesSpecificValue( p_oElement, "tr_stdAsympCrownRad", "tr_sacrVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fAsympCrownRad[p_fTemp[i].code] = p_fTemp[i].val;
      }

      FillSpeciesSpecificValue( p_oElement, "tr_stdCrownRadExp", "tr_screVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCrownRadExp[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //If any of the functions are Chapman-Richards, read parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( chapman_richards_cr == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( linear == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare the arrays we'll need
      if (!mp_fCRCrownRadIntercept)
        mp_fCRCrownRadIntercept = new float[m_iNumSpecies];
      if (!mp_fCRAsympCrownRad)
        mp_fCRAsympCrownRad = new float[m_iNumSpecies];
      if (!mp_fCRCrownRadShape1)
        mp_fCRCrownRadShape1 = new float[m_iNumSpecies];
      if (!mp_fCRCrownRadShape2)
        mp_fCRCrownRadShape2 = new float[m_iNumSpecies];

      //Crown radius intercept
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownRadIntercept", "tr_crcriVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRCrownRadIntercept[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Asymptotic crown radius
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownRadAsymp", "tr_crcraVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRAsympCrownRad[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Shape parameter 1
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownRadShape1b", "tr_crcrs1bVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRCrownRadShape1[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Shape parameter 2
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownRadShape2c", "tr_crcrs2cVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRCrownRadShape2[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //***********************
    // Saplings - crown height-height relationship
    //***********************
    FillSpeciesSpecificValue( p_oElement, "tr_whatSaplingCrownHeightHeight", "tr_wschhVal", p_iWhatFunction, p_oPop, true );

    //Use the values to set function pointers - if any are not a valid function
    //enum number, throw an error
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard_cr == p_iWhatFunction[i] )
      {
        mp_SaplingCrownHeight[i] = & clAllometry::CalcStandardSapAdultCrownHeight;
      }
      else if ( chapman_richards_cr == p_iWhatFunction[i] )
      {
        mp_SaplingCrownHeight[i] = & clAllometry::CalcChapRichSapAdultCrownHeight;
      }
      else
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clAllometry::GetData" );
        sprintf( stcErr.cMoreInfo, "%s%d", "Invalid sapling crown height-diam function code: ", p_iWhatFunction[i] );
        throw( stcErr );
      }
    }

    //If any of the functions are standard, read standard parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard_cr == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( standard == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare arrays we'll need
      if (!mp_fAsympCrownHeight)
        mp_fAsympCrownHeight = new float[m_iNumSpecies];
      if (!mp_fCrownHeightExp)
        mp_fCrownHeightExp = new float[m_iNumSpecies];

      FillSpeciesSpecificValue( p_oElement, "tr_stdAsympCrownHt", "tr_sachVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fAsympCrownHeight[p_fTemp[i].code] = p_fTemp[i].val;
      }

      FillSpeciesSpecificValue( p_oElement, "tr_stdCrownHtExp", "tr_scheVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCrownHeightExp[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //If any of the functions are Chapman-Richards, read parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( chapman_richards_cr == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( linear == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare any arrays we'll need
      if (!mp_fCRCrownHtIntercept)
        mp_fCRCrownHtIntercept = new float[m_iNumSpecies];
      if (!mp_fCRAsympCrownHt)
        mp_fCRAsympCrownHt = new float[m_iNumSpecies];
      if (!mp_fCRCrownHtShape1)
        mp_fCRCrownHtShape1 = new float[m_iNumSpecies];
      if (!mp_fCRCrownHtShape2)
        mp_fCRCrownHtShape2 = new float[m_iNumSpecies];

      //Crown height intercept
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownHtIntercept", "tr_crchiVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRCrownHtIntercept[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Asymptotic crown height
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownHtAsymp", "tr_crchaVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRAsympCrownHt[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Shape parameter 1
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownHtShape1b", "tr_crchs1bVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRCrownHtShape1[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Shape parameter 2
      FillSpeciesSpecificValue( p_oElement, "tr_chRichCrownHtShape2c", "tr_crchs2cVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fCRCrownHtShape2[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //***********************
    // Seedlings
    //***********************
    FillSpeciesSpecificValue( p_oElement, "tr_whatSeedlingHeightDiam", "tr_wsehdVal", p_iWhatFunction, p_oPop, true );

    //Use the values to set function pointers - if any are not a valid function
    //enum number, throw an error
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard == p_iWhatFunction[i] )
      {
        mp_SeedlingHeight[i] = & clAllometry::CalcStandardSeedlingHeight;
        mp_SeedlingDiam[i] = & clAllometry::CalcStandardSeedlingDiam10;
      }
      else if ( linear == p_iWhatFunction[i] )
      {
        mp_SeedlingHeight[i] = & clAllometry::CalcLinearSeedlingHeight;
        mp_SeedlingDiam[i] = & clAllometry::CalcLinearSeedlingDiam10;
      }
      else if ( reverse_linear == p_iWhatFunction[i] )
      {
        mp_SeedlingHeight[i] = & clAllometry::CalcReverseLinearSeedlingHeight;
        mp_SeedlingDiam[i] = & clAllometry::CalcReverseLinearSeedlingDiam10;
      }
      else
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clAllometry::GetData" );
        sprintf( stcErr.cMoreInfo, "%s%d", "Invalid seedling height-diam function code: ", p_iWhatFunction[i] );
        throw( stcErr );
      }
    }

    //If any of the functions are standard, read standard parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( standard == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( standard == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare array we'll need
      mp_fSlopeHeightDiam10 = new float[m_iNumSpecies];

      FillSpeciesSpecificValue( p_oElement, "tr_slopeOfHeight-Diam10", "tr_sohdVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fSlopeHeightDiam10[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //If any of the functions are linear, read linear parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( linear == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( linear == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare arrays we'll need
      mp_fSeedlingLinearSlope = new float[m_iNumSpecies];
      mp_fSeedlingLinearIntercept = new float[m_iNumSpecies];

      //Linear seedling slope - throw error if any values are 0
      FillSpeciesSpecificValue( p_oElement, "tr_seedlingLinearSlope", "tr_selsVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        if ( 0 == p_fTemp[i].val )
        {
          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy( stcErr.cFunction, "clAllometry::GetData" );
          strcpy( stcErr.cMoreInfo, "No values in seedling linear slope may be 0." );
          throw( stcErr );

        }
        mp_fSeedlingLinearSlope[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Linear seedling intercept
      FillSpeciesSpecificValue( p_oElement, "tr_seedlingLinearIntercept", "tr_seliVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fSeedlingLinearIntercept[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //If any of the functions are reverse linear, read linear parameters
    iHowMany = 0;
    for ( i = 0; i < m_iNumSpecies; i++ )
    {
      if ( reverse_linear == p_iWhatFunction[i] ) iHowMany++;
    }
    if ( iHowMany > 0 )
    {
      if ( p_fTemp )
      {
        delete[] p_fTemp; p_fTemp = NULL;
      }
      p_fTemp = new floatVal[iHowMany];
      iHowMany = 0;
      for ( i = 0; i < m_iNumSpecies; i++ )
      {
        if ( reverse_linear == p_iWhatFunction[i] )
        {
          p_fTemp[iHowMany].code = i;
          iHowMany++;
        }
      }

      //Declare arrays we'll need
      mp_fSeedlingReverseLinearSlope = new float[m_iNumSpecies];
      mp_fSeedlingReverseLinearIntercept = new float[m_iNumSpecies];

      //Reverse linear seedling slope - throw an error if any are 0
      FillSpeciesSpecificValue( p_oElement, "tr_seedlingReverseLinearSlope", "tr_serlsVal", p_fTemp, iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        if ( 0 == p_fTemp[i].val )
        {
          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy( stcErr.cFunction, "clAllometry::GetData" );
          strcpy( stcErr.cMoreInfo, "No values in seedling reverse linear slope may be 0." );
          throw( stcErr );
        }
        mp_fSeedlingReverseLinearSlope[p_fTemp[i].code] = p_fTemp[i].val;
      }

      //Reverse linear seedling intercept
      FillSpeciesSpecificValue( p_oElement, "tr_seedlingReverseLinearIntercept", "tr_serliVal", p_fTemp,
           iHowMany, p_oPop, true );
      for ( i = 0; i < iHowMany; i++ )
      {
        mp_fSeedlingReverseLinearIntercept[p_fTemp[i].code] = p_fTemp[i].val;
      }
    }

    //***********************
    // Calculate max canopy radius
    //***********************
    m_fMaxCrownRad = 0;
    float fTempRad = 0;
    for (i = 0; i < m_iNumSpecies; i++) {
      if (& clAllometry::CalcStandardSapAdultCrownRad == mp_AdultCrownRad[i])
        fTempRad = m_fMaxStdCrownRad;
      else if (& clAllometry::CalcChapRichSapAdultCrownRad == mp_AdultCrownRad[i])
        fTempRad = mp_fCRAsympCrownRad[i] + mp_fCRCrownRadIntercept[i];

      if (m_fMaxCrownRad < fTempRad) m_fMaxCrownRad = fTempRad;
    }
    for (i = 0; i < m_iNumSpecies; i++) {
      if (& clAllometry::CalcStandardSapAdultCrownRad == mp_SaplingCrownRad[i])
        fTempRad = m_fMaxStdCrownRad;
      else if (& clAllometry::CalcChapRichSapAdultCrownRad == mp_SaplingCrownRad[i])
        fTempRad = mp_fCRAsympCrownRad[i] + mp_fCRCrownRadIntercept[i];

      if (m_fMaxCrownRad < fTempRad) m_fMaxCrownRad = fTempRad;
    }

    delete[] p_iWhatFunction;
    delete[] p_fTemp;
  }
  catch ( modelErr & err )
  {
    delete[] p_iWhatFunction;
    delete[] p_fTemp;
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    delete[] p_iWhatFunction;
    delete[] p_fTemp;
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    delete[] p_iWhatFunction;
    delete[] p_fTemp;
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clAllometry::GetData" );
    throw( stcErr );
  }
}

////////////////////////////////////////////////////////////////////////////
// CalcAdultHeight()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// April 25, 2005 - Rewrote to use function pointers (LEM)
////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcAdultHeight( const float & fDbh, const int & iSpecies )
{
  if ( iSpecies >= m_iNumSpecies || iSpecies < 0 )
  {
    //invalid species - throw an error
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcAdultHeight" );
    sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
    throw( stcErr );
  }
  return (* this.*mp_AdultHeight[iSpecies] ) ( fDbh, iSpecies );
}

////////////////////////////////////////////////////////////////////////////
// CalcAdultDbh()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// April 25, 2005 - Rewrote to use function pointers (LEM)
////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcAdultDbh( float fHeight, const int & iSpecies )
{
  if ( iSpecies >= m_iNumSpecies || iSpecies < 0 )
  {
    //invalid species - throw an error
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcAdultDbh" );
    sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
    throw( stcErr );
  }
  //If the height is greater than or equal to canopy height, throw bad data error
  if ( fHeight >= mp_fMaxTreeHeight[iSpecies] )
  {
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcAdultDbh" );
    sprintf( stcErr.cMoreInfo, "%s%s", "Cannot set tree height greater than ", "maximum canopy height for this species." );
    throw( stcErr );
  }
  return (* this.*mp_AdultDiam[iSpecies] ) ( fHeight, iSpecies );
}

////////////////////////////////////////////////////////////////////////////
// CalcSaplingHeight()
// Edit history:
// -------------
// April 25, 2005 - Created (LEM)
////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcSaplingHeight( const float & fDbh, const int & iSpecies )
{
  if ( iSpecies >= m_iNumSpecies || iSpecies < 0 )
  {
    //invalid species - throw an error
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcSaplingHeight" );
    sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
    throw( stcErr );
  }
  return (* this.*mp_SaplingHeight[iSpecies] ) ( fDbh, iSpecies );
}

////////////////////////////////////////////////////////////////////////////
// CalcSaplingDbh()
// Edit history:
// -------------
// April 25, 2005 - Created (LEM)
////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcSaplingDbh( float fHeight, const int & iSpecies )
{
  if ( iSpecies >= m_iNumSpecies || iSpecies < 0 )
  {
    //invalid species - throw an error
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcSaplingDbh" );
    sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
    throw( stcErr );
  }
  //If the height is greater than or equal to canopy height, throw bad data error
  if ( fHeight >= mp_fMaxTreeHeight[iSpecies] )
  {
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcSaplingDbh" );
    sprintf( stcErr.cMoreInfo, "%s%s", "Cannot set tree height greater than ", "maximum canopy height for this species." );
    throw( stcErr );
  }
  return (* this.*mp_SaplingDiam[iSpecies] ) ( fHeight, iSpecies );
}

////////////////////////////////////////////////////////////////////////////
// CalcAdultCrownRadius()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// July 8, 2005 - Rewrote to use function pointers (LEM)
////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcAdultCrownRadius( const float & fDbh, const int & iSpecies )
{
  if ( iSpecies >= m_iNumSpecies || iSpecies < 0 )
  {
    //invalid species - throw an error
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcAdultCanRadius" );
    sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
    throw( stcErr );
  }
  return (* this.*mp_AdultCrownRad[iSpecies] ) ( fDbh, iSpecies );
}

////////////////////////////////////////////////////////////////////////////
// CalcAdultCrownHeight()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// July 8, 2005 - Rewrote to use function pointers (LEM)
////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcAdultCrownHeight( float fHeight, const int & iSpecies )
{
  if ( iSpecies >= m_iNumSpecies || iSpecies < 0 )
  {
    //invalid species - throw an error
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcAdultCrownHeight" );
    sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
    throw( stcErr );
  }
  return (* this.*mp_AdultCrownHeight[iSpecies] ) ( fHeight, iSpecies );
}

////////////////////////////////////////////////////////////////////////////
// CalcSaplingCrownRadius()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// July 8, 2005 - Rewrote to use function pointers (LEM)
////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcSaplingCrownRadius( const float & fDbh, const int & iSpecies )
{
  if ( iSpecies >= m_iNumSpecies || iSpecies < 0 )
  {
    //invalid species - throw an error
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcSaplingCanRadius" );
    sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
    throw( stcErr );
  }
  return (* this.*mp_SaplingCrownRad[iSpecies] ) ( fDbh, iSpecies );
}

////////////////////////////////////////////////////////////////////////////
// CalcSaplingCrownHeight()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// July 8, 2005 - Rewrote to use function pointers (LEM)
////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcSaplingCrownHeight( float fHeight, const int & iSpecies )
{
  if ( iSpecies >= m_iNumSpecies || iSpecies < 0 )
  {
    //invalid species - throw an error
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcSaplingCrownHeight" );
    sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
    throw( stcErr );
  }
  return (* this.*mp_SaplingCrownHeight[iSpecies] ) ( fHeight, iSpecies );
}

////////////////////////////////////////////////////////////////////////////
// CalcSeedlingHeight()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// April 25, 2005 - Rewrote to use function pointers (LEM)
////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcSeedlingHeight( float fDiam10, const int & iSpecies )
{
  if ( iSpecies >= m_iNumSpecies || iSpecies < 0 )
  {
    //invalid species - throw an error
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcSeedlingHeight" );
    sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
    throw( stcErr );
  }
  return (* this.*mp_SeedlingHeight[iSpecies] ) ( fDiam10, iSpecies );
}

////////////////////////////////////////////////////////////////////////////
// CalcSeedlingDiam10()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// April 25, 2005 - Rewrote to use function pointers (LEM)
////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcSeedlingDiam10( float fHeight, const int & iSpecies )
{
  if ( iSpecies >= m_iNumSpecies || iSpecies < 0 )
  {
    //invalid species - throw an error
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcSeedlingDiam10" );
    sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
    throw( stcErr );
  }
  //If the height is greater than or equal to 30.1, throw bad data error
  if ( fHeight >= 30.1 )
  {
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::CalcSeedlingDiam10" );
    sprintf( stcErr.cMoreInfo, "%s%s", "Cannot set tree height greater than ", "maximum tree height for this species." );
    throw( stcErr );
  }
  return (* this.*mp_SeedlingDiam[iSpecies] ) ( fHeight, iSpecies );
}

////////////////////////////////////////////////////////////////////////////
// ConvertDiam10ToDbh()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
float clAllometry::ConvertDiam10ToDbh( float fDiam10, const int & iSpecies )
{
  if ( iSpecies >= m_iNumSpecies || iSpecies < 0 )
  {
    //invalid species - throw an error
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::ConvertDiam10ToDbh" );
    sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
    throw( stcErr );
  }
  return (fDiam10 * mp_fDbhToDiam10Slope[iSpecies]) + mp_fDbhToDiam10Intercept[iSpecies];
}

////////////////////////////////////////////////////////////////////////////
// ConvertDbhToDiam10()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
float clAllometry::ConvertDbhToDiam10( const float & fDbh, const int & iSpecies )
{
  if ( iSpecies >= m_iNumSpecies || iSpecies < 0 )
  {
    //invalid species - throw an error
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy( stcErr.cFunction, "clAllometry::ConvertDbhToDiam10" );
    sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
    throw( stcErr );
  }
  return (fDbh - mp_fDbhToDiam10Intercept[iSpecies]) / mp_fDbhToDiam10Slope[iSpecies];
}

//////////////////////////////////////////////////////////////////////////////
// GetMaxTreeHeight()
// Edit history :
// -------------
// April 25, 2005 - Moved to here from clTreePopulation (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::GetMaxTreeHeight( int iSpecies )
{
  try
  {
    if ( iSpecies < 0 || iSpecies >= m_iNumSpecies )
    {
      //invalid species - throw an error
      modelErr stcErr;
      stcErr.iErrorCode = BAD_DATA;
      strcpy( stcErr.cFunction, "clTreePopulation::GetMaxTreeHeight" );
      sprintf( stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies );
      throw( stcErr );
    }
    return mp_fMaxTreeHeight[iSpecies];
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clTreePopulation::GetMaxTreeHeight" );
    throw( stcErr );
  }
}

//////////////////////////////////////////////////////////////////////////////
// CalcStandardSapAdultHeight()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcStandardSapAdultHeight(const float &fDbh, const int &iSpecies)
{
  float fHeight = (1.35 + ((mp_fMaxTreeHeight[iSpecies] - 1.35)*
        (1.0-exp((double)(-mp_fSlopeAsympHeight[iSpecies] * fDbh)))));
  if (fHeight > 0.001) return fHeight; else return 0.001;
}

//////////////////////////////////////////////////////////////////////////////
// CalcLinearAdultHeight()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcLinearAdultHeight(const float &fDbh, const int &iSpecies ) {
  float fHeight = fDbh * mp_fAdultLinearSlope[iSpecies] + mp_fAdultLinearIntercept[iSpecies];
  if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
  if (fHeight <= 0.001) return 0.001;
  return fHeight;
}

//////////////////////////////////////////////////////////////////////////////
// CalcReverseLinearAdultHeight()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcReverseLinearAdultHeight(const float &fDbh, const int &iSpecies ) {
  float fHeight = (fDbh - mp_fAdultReverseLinearIntercept[iSpecies]) / mp_fAdultReverseLinearSlope[iSpecies];
  if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
  if (fHeight <= 0.001) return 0.001;
  return fHeight;
}

//////////////////////////////////////////////////////////////////////////////
// CalcLinearSaplingHeight()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcLinearSaplingHeight(const float &fDbh, const int &iSpecies ) {
  float fHeight = fDbh * mp_fSaplingLinearSlope[iSpecies] + mp_fSaplingLinearIntercept[iSpecies];
  if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
  if (fHeight <= 0.001) return 0.001;
  return fHeight;
}

//////////////////////////////////////////////////////////////////////////////
// CalcReverseLinearSaplingHeight()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcReverseLinearSaplingHeight(const float &fDbh, const int &iSpecies ) {
  float fHeight = (fDbh - mp_fSaplingReverseLinearIntercept[iSpecies]) / mp_fSaplingReverseLinearSlope[iSpecies];
  if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
  if (fHeight <= 0.001) return 0.001;
  return fHeight;
}

//////////////////////////////////////////////////////////////////////////////
// CalcStandardSeedlingHeight()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcStandardSeedlingHeight( const float &fDiam10, const int &iSpecies ) {
  float fHeight = 0.1 + 30*(1.0-exp((double)
             (-mp_fSlopeHeightDiam10[iSpecies]*fDiam10)));
  if (fHeight > 0.001) return fHeight; else return 0.001;
}

//////////////////////////////////////////////////////////////////////////////
// CalcLinearSeedlingHeight()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcLinearSeedlingHeight(const float &fDiam10, const int &iSpecies ) {
  float fHeight = fDiam10 * mp_fSeedlingLinearSlope[iSpecies] + mp_fSeedlingLinearIntercept[iSpecies];
  if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
  if (fHeight <= 0.001) return 0.001;
  return fHeight;
}

//////////////////////////////////////////////////////////////////////////////
// CalcReverseLinearSeedlingHeight()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcReverseLinearSeedlingHeight(const float &fDiam10, const int &iSpecies ) {
  float fHeight = (fDiam10 - mp_fSeedlingReverseLinearIntercept[iSpecies]) / mp_fSeedlingReverseLinearSlope[iSpecies];
  if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
  if (fHeight <= 0.001) return 0.001;
  return fHeight;
}

//////////////////////////////////////////////////////////////////////////////
// CalcStandardSapAdultDbh()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcStandardSapAdultDbh( const float &fHeight, const int &iSpecies ) {
  return log(1-((fHeight-1.35)/(mp_fMaxTreeHeight[iSpecies]-1.35)))/
  (-mp_fSlopeAsympHeight[iSpecies]);
}

//////////////////////////////////////////////////////////////////////////////
// CalcLinearAdultDbh()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcLinearAdultDbh(const float &fHeight, const int &iSpecies ) {
  return (fHeight - mp_fAdultLinearIntercept[iSpecies]) / mp_fAdultLinearSlope[iSpecies];
}

//////////////////////////////////////////////////////////////////////////////
// CalcReverseLinearAdultDbh()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcReverseLinearAdultDbh(const float &fHeight, const int &iSpecies ) {
  return fHeight * mp_fAdultReverseLinearSlope[iSpecies] + mp_fAdultReverseLinearIntercept[iSpecies];
}

//////////////////////////////////////////////////////////////////////////////
// CalcLinearSaplingDbh()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcLinearSaplingDbh(const float &fHeight, const int &iSpecies ) {
  return (fHeight - mp_fSaplingLinearIntercept[iSpecies]) / mp_fSaplingLinearSlope[iSpecies];
}

//////////////////////////////////////////////////////////////////////////////
// CalcReverseLinearSaplingDbh()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcReverseLinearSaplingDbh(const float &fHeight, const int &iSpecies ) {
  return fHeight * mp_fSaplingReverseLinearSlope[iSpecies] + mp_fSaplingReverseLinearIntercept[iSpecies];
}

//////////////////////////////////////////////////////////////////////////////
// CalcStandardSeedlingDiam10()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcStandardSeedlingDiam10( const float &fHeight, const int &iSpecies ) {
  return (log(1-((fHeight-0.1)/30)))/-mp_fSlopeHeightDiam10[iSpecies];
}

//////////////////////////////////////////////////////////////////////////////
// CalcLinearSeedlingDiam10()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcLinearSeedlingDiam10(const float &fHeight, const int &iSpecies ) {
  return (fHeight - mp_fSeedlingLinearIntercept[iSpecies]) / mp_fSeedlingLinearSlope[iSpecies];
}

//////////////////////////////////////////////////////////////////////////////
// CalcReverseLinearSeedlingDiam10()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcReverseLinearSeedlingDiam10(const float &fHeight, const int &iSpecies ) {
  return fHeight * mp_fSeedlingReverseLinearSlope[iSpecies] + mp_fSeedlingReverseLinearIntercept[iSpecies];
}

//////////////////////////////////////////////////////////////////////////////
// CalcStandardSapAdultCrownRad()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcStandardSapAdultCrownRad(const float &fDbh, const int &iSpecies) {
  float fRad = mp_fAsympCrownRad[iSpecies] * pow( fDbh, mp_fCrownRadExp[iSpecies] );
  return fRad > m_fMaxStdCrownRad ? m_fMaxStdCrownRad : fRad;
}

//////////////////////////////////////////////////////////////////////////////
// CalcChapRichSapAdultCrownRad()
// Edit history :
// -------------
// July 8, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcChapRichSapAdultCrownRad(const float &fDbh, const int &iSpecies) {
  return mp_fCRCrownRadIntercept[iSpecies] + (mp_fCRAsympCrownRad[iSpecies] * (pow((1-exp(-mp_fCRCrownRadShape1[iSpecies] * fDbh)), mp_fCRCrownRadShape2[iSpecies])));
}

//////////////////////////////////////////////////////////////////////////////
// CalcStandardSapAdultCrownHeight()
// Edit history :
// -------------
// April 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcStandardSapAdultCrownHeight(const float &fHeight, const int &iSpecies) {
  float fCrownHeight = mp_fAsympCrownHeight[iSpecies] * pow( fHeight, mp_fCrownHeightExp[iSpecies] );
  return fCrownHeight > fHeight ? fHeight : fCrownHeight;
}

//////////////////////////////////////////////////////////////////////////////
// CalcChapRichSapAdultCrownHeight()
// Edit history :
// -------------
// July 8, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcChapRichSapAdultCrownHeight(const float &fHeight, const int &iSpecies) {
  float fCHeight = mp_fCRCrownHtIntercept[iSpecies] + (mp_fCRAsympCrownHt[iSpecies] * (pow((1-exp(-mp_fCRCrownHtShape1[iSpecies] * fHeight)), mp_fCRCrownHtShape2[iSpecies])));
  return (fCHeight > fHeight) ? fHeight : fCHeight;
}

//////////////////////////////////////////////////////////////////////////////
// CalcPowerSaplingHeight()
// Edit history :
// -------------
// January 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcPowerSaplingHeight(const float &fDbh, const int &iSpecies ) {
  if (fDbh <= 0.001) return 0.001;
  //Turn DBH into diam10
  float fDiam10 = ConvertDbhToDiam10(fDbh, iSpecies),
        fHeight = mp_fPowerA[iSpecies] * pow(fDiam10, mp_fPowerExpB[iSpecies]);
  if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
  if (fHeight <= 0.001) return 0.001;
  return fHeight;
}

//////////////////////////////////////////////////////////////////////////////
// CalcPowerSaplingDbh()
// Edit history :
// -------------
// January 25, 2005 - Created (LEM )
//////////////////////////////////////////////////////////////////////////////
float clAllometry::CalcPowerSaplingDbh(const float &fHeight, const int &iSpecies ) {
  if (fHeight <= 0.001) return 0.001;
  float fDiam = pow(fHeight / mp_fPowerA[iSpecies], 1/mp_fPowerExpB[iSpecies]);
  //Convert to DBH
  return ConvertDiam10ToDbh(fDiam, iSpecies);
}

