//---------------------------------------------------------------------------
#include "ConstantBAGrowth.h"
#include "Tree.h"
#include "GrowthOrg.h"
#include "ParsingFunctions.h"
#include "SimManager.h"
#include "TreePopulation.h"

//////////////////////////////////////////////////////////////////////////////
// clConstantBAGrowth()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
//////////////////////////////////////////////////////////////////////////////
clConstantBAGrowth::clConstantBAGrowth(clSimManager *p_oSimManager) :
    clWorkerBase(p_oSimManager), clBehaviorBase(p_oSimManager), clGrowthBase(p_oSimManager),
    clMichMenBase(p_oSimManager) {
  strcpy(m_cNameString, "constbagrowthshell");
}

//////////////////////////////////////////////////////////////////////////////
// CalcDiameterGrowthValue()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
//////////////////////////////////////////////////////////////////////////////
float clConstantBAGrowth::CalcDiameterGrowthValue(clTree *p_oTree,
                    clTreePopulation *p_oPop, float fHeightGrowth) {
 try {
   float fDiam;                   //tree's diameter value

   //Get the appropriate diameter for this tree
   p_oTree->GetValue(mp_oGrowthOrg->GetDiamCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fDiam);

   return mp_fAdultConstBAInc[p_oTree->GetSpecies()] / fDiam;
 }
 catch (modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clConstantBAGrowth::CalcGrowthValue");
   throw(stcErr);
 }
}
//---------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////////////
// SetNameData()
// Edit history:
// -------------
// May 21, 2004 - Created (to support diameter-only growth) (LEM)
/////////////////////////////////////////////////////////////////////////////*/
void clConstantBAGrowth::SetNameData(char *cNameString) {
 try {

   //Check the string passed and set the flags accordingly
   if (strcmp("constbagrowth", cNameString) == 0) {
     m_iGrowthMethod = diameter_auto;
   } else if (strcmp("constbagrowth diam only", cNameString) == 0){
     m_iGrowthMethod = diameter_only;
   } else {
     modelErr stcErr;
     stcErr.iErrorCode = BAD_DATA;
     sprintf(stcErr.cMoreInfo, "%s%s%s", "Unrecognized behavior name \"",
        cNameString, "\".");
     strcpy(stcErr.cFunction, "clConstantBAGrowth::SetNameData");
     throw(stcErr);
   }
 }
 catch (modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clConstantBAGrowth::SetNameData");
   throw(stcErr);
 }
}
//---------------------------------------------------------------------------

/*/////////////////////////////////////////////////////////////////////////////
DoShellSetup()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////*/
void clConstantBAGrowth::DoShellSetup(xercesc::DOMDocument *p_oDoc) {
 try {
   short int iNumSpecies = mp_oGrowthOrg->GetNumberOfSpecies();

   mp_fAdultConstBAInc = new float[iNumSpecies];

   //Read the base variables
   GetParameterFileData(p_oDoc);
 }
 catch (modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clRelativeGrowth::GetNameData");
   throw(stcErr);
 }
}
//---------------------------------------------------------------------------
