//---------------------------------------------------------------------------
// GrowthHeightIncrementer.cpp
//---------------------------------------------------------------------------
#include "GrowthHeightIncrementer.h"
#include "SimManager.h"
#include "TreePopulation.h"
#include "Allometry.h"
#include "GrowthOrg.h"

//////////////////////////////////////////////////////////////////////////////
// Constructor
// Edit history:
// -------------
// May 21, 2004 - Created (LEM)
/////////////////////////////////////////////////////////////////////////////*/
clGrowthHeightIncrementer::clGrowthHeightIncrementer( clSimManager * p_oSimManager ) : 
     clWorkerBase( p_oSimManager ), clBehaviorBase( p_oSimManager ), clGrowthBase( p_oSimManager )
     {

       try
       {
         strcpy( m_cNameString, "height incrementer growthshell" );
         m_iGrowthMethod = height_only;
         m_bGoLast = true;
       }
       catch ( modelErr & err )
       {
         throw( err );
       }
       catch ( modelMsg & msg )
       {
         throw( msg );
       } //non-fatal error
       catch ( ... )
       {
         modelErr stcErr;
         stcErr.iErrorCode = UNKNOWN;
         strcpy( stcErr.cFunction, "clGrowthHeightIncrementer::clGrowthHeightIncrementer" );
         throw( stcErr );
       }
}

//---------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////////////
// DoShellSetup
// Edit history:
// -------------
// May 21, 2004 - Created (LEM)
/////////////////////////////////////////////////////////////////////////////*/
void clGrowthHeightIncrementer::DoShellSetup( DOMDocument * p_oDoc )
{
  clTreePopulation * p_oPop = ( clTreePopulation * ) mp_oSimManager->GetPopulationObject( "treepopulation" );

  //Get the allometry object
  mp_oAllom = p_oPop->GetAllometryObject();
}

//---------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////////////
// CalcHeightGrowthValue
// Edit history:
// -------------
// May 21, 2004 - Created (LEM)
/////////////////////////////////////////////////////////////////////////////*/
float clGrowthHeightIncrementer::CalcHeightGrowthValue( clTree * p_oTree, clTreePopulation * p_oPop, float fDiameterGrowth )
{
  try
  {
    float fBeginningDiam, //diameter before growth is applied
         fEndDiam, //diameter after growth is applied
         fBeginningHeight, //height for beginning diameter
         fEndHeight, //height for end diameter
         fHeightInc; //height increment to return
    int iTp = p_oTree->GetType(), iSp = p_oTree->GetSpecies();

    //Get the beginning and ending diameters
    p_oTree->GetValue(mp_oGrowthOrg->GetDiamCode(iSp, iTp), &fBeginningDiam);
    fEndDiam = fBeginningDiam + fDiameterGrowth;

    //Get the beginning and ending height according to tree type
    if (clTreePopulation::seedling == iTp) {
      fBeginningHeight = mp_oAllom->CalcSeedlingHeight(fBeginningDiam, iSp);
      fEndHeight = mp_oAllom->CalcSeedlingHeight(fEndDiam, iSp);
    }
    else if (clTreePopulation::sapling == iTp) {
      //Transform these diameters to DBH
      fBeginningDiam = mp_oAllom->ConvertDiam10ToDbh(fBeginningDiam, iSp);
      fEndDiam = mp_oAllom->ConvertDiam10ToDbh(fEndDiam, iSp);
      fBeginningHeight = mp_oAllom->CalcSaplingHeight(fBeginningDiam, iSp);
      fEndHeight = mp_oAllom->CalcSaplingHeight(fEndDiam, iSp);
    } else {
      fBeginningHeight = mp_oAllom->CalcAdultHeight(fBeginningDiam, iSp);
      fEndHeight = mp_oAllom->CalcAdultHeight(fEndDiam, iSp);
    }

    fHeightInc = fEndHeight - fBeginningHeight;
    return fHeightInc;
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clGrowthHeightIncrementer::CalcHeightGrowthValue" );
    throw( stcErr );
  }

}
//---------------------------------------------------------------------------




