//---------------------------------------------------------------------------

#ifndef TreeRemoverH
#define TreeRemoverH
//---------------------------------------------------------------------------
#include "BehaviorBase.h"

class clSimManager;
class xercesc::DOMDocument;
/**
  Tree Killer - Version 2.0

  This behavior will remove all trees that have been marked for death by
  mortality.  If a tree has a value of true in its "dead" data member,
  this will direct it to be removed from the tree population by calling
  clTreePopulation::KillTree().  The reason code will always be set to
  "natural".

  If this is applied to any species/type combo that does not have "dead"
  registered, an error will be thrown during setup.

  The namestring and parameter call string for this behavior are both
  "removedead".

  Copyright 2003 Charles D. Canham.
  @author Lora E. Murphy

  <br>Edit history:
  <br>-----------------
  <br>April 28, 2004 - Submitted as beta, as clLGMTreeKiller (LEM)
  <br>April 29, 2004 - Released version 2 (LEM) - based on mortality's tree
                       data member instead of light's - renamed this class
*/
class clTreeRemover : virtual public clBehaviorBase {

  public:

/**
  Constructor.

  @param p_oSimManager Sim Manager object.
*/
  clTreeRemover(clSimManager *p_oSimManager);

/**
  Destructor.
*/
  ~clTreeRemover();

/**
* Does all the tree killin'.  This goes through all trees in the tree
* population to which this behavior is applied, and if any have true as the
* value in their "dead" data member, their "dead" data member is set to false
* and they are passed to clTreePopulation::KillTree() with a reason code of
* "natural".
*/
  void Action();

/**
* Gets the data member codes for the "dead" data member.
* @param p_oDoc DOM tree of parsed input file.
* @throws modelErr if a tree type/species combo to which this behavior is
* applied has no "dead" code.
*/
  void GetData(xercesc::DOMDocument *p_oDoc);

  protected:
  clTreePopulation *mp_oPop; /**<Stashed pointer to avoid having to keep getting it*/

  /**
  Return codes for the "dead" tree bool data member variable.  Array size is
  species by type (even if not every species and type is represented)*/
  short int **mp_iDeadCodes;
  short int m_iNumTotalSpecies; /**<Total number of species - for destructor*/
};

//---------------------------------------------------------------------------
#endif
